'*******************************************************************************
' Module      : ASCII Set 32-255
' Author      : Terence Lim
' Created     : 15/12/2008 2:27:08 PM
' Last Update : 16/12/2008 2:27:12 PM
' Purpose     :
'*******************************************************************************

Const module_name = "JulianDates"
Const module_ver = "1.00"   

Const mbPrompt = "Script Cancelled or Error Encountered"
Const mbTitle = "Cancel or Error"

' Sample Dates
' ============
' 20/08/2002 - 02232 - Normal Date to Julian
' 27/11/2008 - 08332 - Normal Date to Julian
' 02232 - 20/08/2002 - Julian to Normal Date
' 08332 - 27/11/2008 - Julian to Normal Date
' 37488 - 02232 - Serial Date to Julian
' 39779 - 08332 - Serial Date to Julian
' 02232 - 37488 - Julian to Serial Date
' 08332 - 39779 - Julian to Serial Date 

'This procedure takes a normal date format (that is, 1/1/94) 
'and converts it to the appropriate Julian date.
Sub NormalToJulian()
    Dim CRLF           'Carriage return/line feed.
    Dim NormalDate       'The serial date.
    Dim DateYear       'The year of the serial date.
    Dim JulianDay 
    Dim JulianDate     'The converted Julian date value

    'Assign CRLF the value of a carriage return and line feed.
    CRLF = Chr(13)
   
      '// Get working text
  	NormalDate = handleSelText("")
    
    If isDate(NormalDate) = False or isNull(NormalDate) = True or NormalDate = "" or isEmpty(NormalDate) = True Then 
			 msgbox mbPrompt, vbCritical + vbOkOnly, mbTitle 
			 Exit Sub
		End If	 

    NormalDate = DateValue(NormalDate)

    'Assign DateYear the year number
    DateYear = Right(Year(NormalDate),2)

    'Find the day number for NormalDate
    JulianDay = PadTo(CStr(NormalDate - DateValue("1/1/" & CStr(DateYear)) + 1), 3)

    'Combine the year and day to get the value for JulianDate.
    JulianDate = DateYear & JulianDay

    'Display the new date in the Julian date format.
'    MsgBox "The equivalent Julian date is " & JulianDate 'Uncomment to have function display a messagebox
    setClipboardText(JulianDate)
    logClear()
    logAddLine("The equivalent Julian Date is " & JulianDate)
End Sub

'The following formula converts a Julian date to a standard
'date (the formula assumes that your date is in the 20th century)
Sub JulianToNormal()
  'Holds the value of the string entered in the Input Box.
  Dim JulianDate 
  'The converted normal date value.
  Dim NormalDate 

  '// Get working text
  JulianDate = handleSelText("")
  
  If isNumeric(JulianDate) = False or isNull(JulianDate) = True or JulianDate = "" or isEmpty(JulianDate) = True Then
		 msgbox mbPrompt, vbCritical + vbOkOnly, mbTitle 
		 Exit Sub
	End If
		 
  JulianDate = CLng(JulianDate)

  'Convert the Julian date to a serial date.
  NormalDate = DateSerial(CLng(JulianDate / 1000), 1, _
     JulianDate Mod 1000)

  'Display the new date in the normal date format.
'   MsgBox "The equivalent date is " & UKDate(NormalDate)  'Uncomment to have function display a messagebox
  setClipboardText(UKDate(NormalDate))
  logClear()
  logAddLine("The equivalent Normal Date is " & UKDate(NormalDate))
End Sub

'This procedure takes a serial date number entered 
'and converts it to the appropriate Julian date and returns it as a
'string.
Sub SerialToJulian()
    Dim SerialDate       'The serial date.
    Dim SerialYear     'The year of the serial date.
    Dim JulianDay 
    Dim JulianDate     'The converted Julian date value

	  '// Get working text
	  SerialDate = handleSelText("")
    
    If isNumeric(SerialDate) = False or isNull(SerialDate) = True or SerialDate = "" or isEmpty(SerialDate) = True Then
			 msgbox mbPrompt, vbCritical + vbOkOnly, mbTitle 
			 Exit Sub
		End If	 
    
		SerialDate = CLng(SerialDate)
		
    'Assign SerialYear the year number
    SerialYear = Right(Year(SerialDate),2)
		    
'    JulianDay = PadTo(CStr(SerialDate - DateSerial(dYr, dMth, dDay)), 3)
    JulianDay = PadTo(CStr(CDate(SerialDate) - DateValue("1/1/" & CStr(SerialYear)) + 1), 3)
        
    JulianDate = SerialYear & JulianDay

    'Display the new date in the normal date format.
'    MsgBox "The equivalent Julian date is " & JulianDate  'Uncomment to have function display a messagebox
    setClipboardText(JulianDate)
    logClear()
    logAddLine("The equivalent Julian Date is " & JulianDate)
End Sub

'The following formula converts a Julian date to a standard serial
'number date (the formula assumes that your date is in the 20th century)
Sub JulianToSerial()
  'Holds the value of the string entered in the Input Box.
  Dim JulianDate 
  'The converted serial date value.
  Dim SerialDate 

  '// Get working text
  JulianDate = handleSelText("")
  
  If isNumeric(JulianDate) = False or isNull(JulianDate) = True or JulianDate = "" or isEmpty(JulianDate) = True Then
		 msgbox mbPrompt, vbCritical + vbOkOnly, mbTitle 
		 Exit Sub
	End If
		 
  JulianDate = CLng(JulianDate)
  
  'Convert the Julian date to a serial date.
  SerialDate = DateSerial(CLng(JulianDate / 1000), 1, _
     JulianDate Mod 1000)

  'Find the day number for SerialDate
	SerialDate = CLng(CDate(SerialDate))
			
  'Display the new date in the normal date format.
'   MsgBox "The equivalent Serial Date is " & SerialDate  'Uncomment to have function display a messagebox
  setClipboardText(SerialDate)
  logClear()
  logAddLine("The equivalent Serial Date is " & SerialDate)
End Sub

Private Function UKDate(dSerialDate)

		dDay = PadTo(DatePart("d", dSerialDate), 2)
		dMth = PadTo(DatePart("m", dSerialDate), 2)
		dYr = DatePart("yyyy", dSerialDate)
		UKDate = dDay & "/" & dMth & "/" & dYr

End Function

Private Function PadTo(strPad, intPadLen)

If Len(strPad) < intPadLen Then 
    PadTo = String(intPadLen - Len(strPad), "0") & strPad
  else
	  PadTo = strPad
End If

End Function

'// @param string Text to replace selected text
Private Function handleSelText(text)
    On Error Resume Next
    Set editor = newEditor()
    editor.assignActiveEditor
    If text = "" Then
        '// Get selected text
        handleSelText = editor.selText
        If handleSelText = "" Then
            '// No text was select. Get all text and select it.
            handleSelText  = editor.Text
            editor.command "ecSelectAll"
        End If
    Else
        '// Set selected text
        editor.selText text
    End If
End Function

Sub Init
    addMenuItem "Standard -> Julian", "Julian Dates", "NormalToJulian"
    addMenuItem "Serial -> Julian", "Julian Dates", "SerialToJulian"
    addMenuItem "Julian -> Standard", "Julian Dates", "JulianToNormal"
		addMenuItem "Julian -> Serial", "Julian Dates", "JulianToSerial"
End Sub
